//! Shared models between server and client code.

use serde::Deserialize;
use serde::Serialize;
use std::collections::BTreeMap;

#[derive(Serialize, Deserialize, Clone, Debug, Default, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct HoldPosition {
    /// Starting from 0
    pub row: u64,

    /// Starting from 0
    pub col: u64,
}

#[derive(Serialize, Deserialize, Clone, Debug, Default, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct Problem {
    pub holds: BTreeMap<HoldPosition, HoldRole>,
}

/// The role of a hold on a route
#[derive(Serialize, Deserialize, Clone, Debug, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub enum HoldRole {
    /// Start hold
    Start,

    /// Any hold on the route without a specific role
    Normal,

    /// Zone hold
    Zone,

    /// End hold
    End,
}
